#ifndef GUI_COMBOBOX_H
#define GUI_COMBOBOX_H

#include "gui_element.h"
#include "app_workspace.h"

/**
 * @brief Wrapper for ImGui combo box structure.
 * 
 */
class gui_combobox : public gui_element {
    private:
        app_workspace_ns::font_size font_size = app_workspace_ns::font_size::NORMAL_FONT;
        int si = 0, list_size = 0; // si = selected_index (internal use only)
        int *selected_index;
        const char **list = nullptr;
        const char *label = nullptr;
        const char *selected_item = nullptr;
        /** When the value is changed, this listener can be called */
        void (*callback)() = nullptr;

    public:
        /**
         * @brief Construct a new gui combobox object
         * 
         * @param list is an array of labels, that the combobox selection consists of
         * @param list_size label array size
         * @param selected_index this variable contains selected index
         * @param label combox can have a label (name), nullptr when not used
         * @param x if set to -1, isn't used
         * @param y if set to -1, isn't used
         * @param width if set to -1, isn't used
         * @param font_size Default font size is NORMAL
         */
        gui_combobox(const char** list, int list_size, int *selected_index, const char *label = nullptr,
                     int x = -1, int y = -1, int width = -1,
                     app_workspace_ns::font_size font_size = app_workspace_ns::font_size::NORMAL_FONT);
        /** Override of parent function, that is required, because it does the actual rendering of the element */
        void render_element();
        /** Returns selected index, that is used internally. This is for legacy reasons, 
         * only selected_index could be used */
        int get_selected_index();
        /** Returns selected label */
        const char* get_selected_item();
        /** Sets value change action */
        void set_value_change_action(void (*callback)());
};

#endif