#ifndef GUI_OBSERVER_H
#define GUI_OBSERVER_H

#include "gui_element.h"
#include "app_workspace.h"

/**
 * @brief Types, that can be observed
 * 
 */
enum observed_type {
    OBS_INT,
    OBS_LONG,
    OBS_FLOAT,
    OBS_STRING
};

/**
 * @brief Allows to create a label, that automatically updates to observed variable value.
 * This class doesn't work well with strings.
 * 
 */
class gui_observer : public gui_element {
    private:
        observed_type type;
        void* target;
        const char *label = nullptr;
        app_workspace_ns::font_size font_size = app_workspace_ns::font_size::NORMAL_FONT;

    public:
        /**
         * @brief Construct a new gui observer object
         * 
         * @param type Observed type
         * @param target Observed data
         * @param x if set to -1, isn't used
         * @param y if set to -1, isn't used
         * @param width if set to -1, isn't used
         * @param font_size Default font size is NORMAL
         */
        gui_observer(observed_type type, void* target, int x = -1, int y = -1, int width = -1,
                app_workspace_ns::font_size font_size = app_workspace_ns::font_size::NORMAL_FONT);

        /** Override of parent function, that is required, because it does the actual rendering of the element */
        void render_element();
};

#endif