#ifndef GUI_SELECTABLE_H
#define GUI_SELECTABLE_H

#include <string>
#include "gui_element.h"
#include "imgui.h"

/**
 * @brief This creates an ImGui::Seletable, which can serve as a label that can be navigated to
 * 
 */
class gui_selectable : public gui_element {
    private:
        int *selector = nullptr;
        int index = -1;
        const char *label = nullptr;
        int height = -1;
        void (*callback)() = nullptr;
        ImGuiSelectableFlags flags = 0;
        std::string inner_label;

    public:
        /**
         * @brief Construct a new gui selectable object
         * 
         * @param label of the item
         * @param index index of this selectable, because it is expected that this is used on an array of values
         * @param selector variable that contains selected item
         * @param callback action that is taken when selected item is activated
         * @param width if set to -1, isn't used
         * @param height if set to -1, isn't used
         * @param flags Flags modifying the element behaviour
         */
        gui_selectable(const char *label, int index, int *selector, void (*callback)(),
                       int width = -1, int height = -1, ImGuiSelectableFlags flags = 0);

        void render_element();
};

#endif