#ifndef SCREEN_MANAGER_H
#define SCREEN_MANAGER_H

#include <stdint.h>
#include <unordered_map>
#include <vector>
#include <memory>
#include "gui_element.h"

#define PREV_SCREEN_Q_MAX 50

#define LOGIN_SCREEN        1
#define MEASURING_SCREEN    2
#define SELECTION_SCREEN    3
#define DETAIL_SCREEN       4
#define ADMIN_SELECT_SCREEN 5
#define ADMIN_CTRL_SCREEN   6

class screen_manager {
    private:
        uint8_t selected_screen = 1;
        //std::map<uint8_t, std::unique_ptr<screen_interface>> available_screens;
        std::unordered_map<uint8_t, std::vector<std::unique_ptr<gui_element>>> available_screens;
        std::vector<uint8_t> prev_screen_stack; // FIXME - deque might be a bit more efficient

        void render_screen(std::vector<std::unique_ptr<gui_element>> &elements);
        bool is_screen_available_to_user(const uint8_t *screens, const uint8_t count, const uint8_t requested);

    public:
        gui_element *focused;
        gui_element *activated;

        screen_manager();
        void render_active_screen();
        // added instead of making selecte_screen public, because setter is being used
        uint8_t get_selected_screen(); 
        void refresh_screen_elements(uint8_t screen);
        void refresh_all_screns();
        void set_selected_screen(uint8_t screen);
        void set_err_screen_content();
        void push_screen_to_previous();
        void pop_screen_from_previous();
        void clear_previous_screen_stack();
        uint8_t get_screen_count();
};

#endif