#ifndef UTILITY_H
#define UTILITY_H

#include <vector>
#include <string>
#include <algorithm> 
#include <cctype>
#include <locale>


/**
 * @brief Simple argument parser
 * 
 * https://stackoverflow.com/questions/865668/parsing-command-line-arguments-in-c
 */
class input_parser {
    public:
        input_parser (int &argc, char **argv);
        /// @author iain
        const std::string& get_cmd_option(const std::string &option) const;
        /// @author iain
        bool cmd_option_exists(const std::string &option) const;
    private:
        std::vector<std::string> tokens;
};

// https://stackoverflow.com/questions/216823/how-to-trim-a-stdstring
/** trim from start (in place) */
static inline void ltrim(std::string &s) {
    s.erase(s.begin(), std::find_if(s.begin(), s.end(), [](unsigned char ch) {
        return !std::isspace(ch);
    }));
}

/** trim from end (in place) */
static inline void rtrim(std::string &s) {
    s.erase(std::find_if(s.rbegin(), s.rend(), [](unsigned char ch) {
        return !std::isspace(ch);
    }).base(), s.end());
}

/** trim from both ends (in place) */
static inline void trim(std::string &s) {
    ltrim(s);
    rtrim(s);
}

#endif