#include "gui_combobox.h"


gui_combobox::gui_combobox(const char** list, int list_size, int *selected_index, const char *label,
                           int x, int y, int width, app_workspace_ns::font_size font_size) {
    this->list = list;
    this->list_size = list_size;
    this->selected_index = selected_index;
    si = *selected_index; // if the screen is refreshed (elements are reinitialized), this should select corrent opt
    this->label = label;
    this->x = x;
    this->y = y;
    this->width = width;
    this->font_size = font_size;
}

void gui_combobox::render_element() {
    gui_element::render_element();

    app_workspace_ns::font_size current_fs = app_workspace::get_instance()->get_font_size();
    char lab[100] = {0};
    const char* preview_value = list[si];

    // if element is requested with different font size
    if (current_fs != font_size)
        app_workspace::get_instance()->use_font_size(font_size);

    if (width > -1)
        ImGui::SetNextItemWidth((float) width);
        //ImGui::PushItemWidth((float) width);

    sprintf(lab, "%s##%d", label ? label : "", index);
    
    // slightly changed version from ImGui Demo
    if (ImGui::BeginCombo(lab, preview_value)) {
        for (int n = 0; n < list_size; n++) {
            const bool is_selected = (si == n);
            if (ImGui::Selectable(list[n], is_selected)) {
                si = n;
                *(this->selected_index) = n;
                selected_item  = list[n];

                if (this->refresh_screen != nullptr)
                    *(this->refresh_screen) = true;
                
                if (this->callback != nullptr)
                    this->callback();
            }

            // Set the initial focus when opening the combo (scrolling + keyboard navigation focus)
            if (is_selected)
                ImGui::SetItemDefaultFocus();
        }
        ImGui::EndCombo();
    }

    // return back original font size (otherwise all subsequent elements would use font_size, unless specified)
    if (current_fs != font_size)
        app_workspace::get_instance()->use_font_size(current_fs);
}

int gui_combobox::get_selected_index() {
    return si;
}

const char* gui_combobox::get_selected_item() {
    return selected_item;
}

void gui_combobox::set_value_change_action(void (*callback)()) {
    this->callback = callback;
}