#include "gui_input.h"

#include <spdlog/spdlog.h>


gui_input::gui_input(input_type type, void *data, int data_size, int x, int y, int width,
                     const char *label, app_workspace_ns::font_size font_size)
{
    this->type = type;
    this->data = data;
    this->data_size = data_size;
    this->x = x;
    this->y = y;
    this->width = width;
    this->label = label;
    this->font_size = font_size;
}

void gui_input::render_element() {
    gui_element::render_element();

    app_workspace_ns::font_size current_fs = app_workspace::get_instance()->get_font_size();
    char lab[100] = {0};
    bool modified = false;

    // if element is requested with different font size
    if (current_fs != font_size)
        app_workspace::get_instance()->use_font_size(font_size);

    if (width > -1)
        ImGui::SetNextItemWidth((float) width);
        //ImGui::PushItemWidth((float) width);

    sprintf(lab, "%s##%d", label ? label : "", index);
    /*
    if (this->label != nullptr)
        sprintf(lab, "%s##%d", this->label, this->index);
    else
        sprintf(lab, "##%d", this->index);
    */

    switch (this->type) {
        case IN_INT:
            modified = ImGui::InputInt(lab, (int*) data, 0, 0);
            break;
        case IN_FLOAT:
            modified = ImGui::InputFloat(lab, (float*) data);
            break;
        case IN_PASSWORD_PLAIN:
        case IN_TEXT:
            //modified = ImGui::InputText(lab, (char*) data, data_size);
            ImGui::InputText(lab, (char*) data, data_size);
            break;
        case IN_PASSWORD:
            //modified = ImGui::InputText(lab, (char*) data, data_size, ImGuiInputTextFlags_Password);
            ImGui::InputText(lab, (char*) data, data_size, ImGuiInputTextFlags_Password);
            break;
        case IN_DOUBLE:
            modified = ImGui::InputDouble(lab, (double*) data);
            break;

        default:
            spdlog::error("gui_input.cpp - Unrecognized input type");
            break;
    }

    // return back original font size (otherwise all subsequent elements would use font_size, unless specified)
    if (current_fs != font_size)
        app_workspace::get_instance()->use_font_size(current_fs);
    
    if (modified) {
        check_input_limit();
        
        if (this->callback != nullptr)
            this->callback();
    }
}


void gui_input::check_input_limit() {
    // TODO/ FIXME - return to last valid value instead of limit?
    switch (this->type) {
        case IN_INT:
            if (*((int*) data) < i_min) *((int*) data) = i_min;
            if (*((int*) data) > i_max) *((int*) data) = i_max;
            break;
        case IN_FLOAT:
            if (*((float*) data) < f_min) *((float*) data) = f_min;
            if (*((float*) data) > f_max) *((float*) data) = f_max;
            break;
        case IN_DOUBLE:
            if (*((double*) data) < d_min) *((double*) data) = d_min;
            if (*((double*) data) > d_max) *((double*) data) = d_max;
            break;

        default:
            spdlog::error("gui_input.cpp - Unrecognized input type of modified value");
            break;
    }   
}

input_type gui_input::get_type() {
    return type;
}


app_workspace_ns::kb_input_type gui_input::get_in_type() {
    // change: password moved to be IN_INT instead of IN_TEXT (due to difficulty inputting with key cycling)
    switch (type) {
        case input_type::IN_TEXT:   return app_workspace_ns::kb_input_type::TEXT;
        case input_type::IN_PASSWORD_PLAIN:
        case input_type::IN_PASSWORD:
        case input_type::IN_INT:        return app_workspace_ns::kb_input_type::INT;
        case input_type::IN_DOUBLE:
        case input_type::IN_FLOAT:      return app_workspace_ns::kb_input_type::FLOAT;
        default:                        return app_workspace_ns::kb_input_type::NONE;
    }
}

void gui_input::set_min_max(int min, int max) {
    this->has_min_max = true;
    this->i_min = min;
    this->i_max = max;
}

void gui_input::set_min_max(float min, float max) {
    this->has_min_max = true;
    this->f_min = min;
    this->f_max = max;
}

void gui_input::set_min_max(double min, double max) {
    this->has_min_max = true;
    this->d_min = min;
    this->d_max = max;
}

void gui_input::set_value_change_action(void (*callback)()) {
    this->callback = callback;
}