#include "localisation.h"

#include <unordered_map>
#include <string>
#include <fstream>
#include <exception>
#include <spdlog/spdlog.h>
#include "app_workspace.h"
#include "utility.h"

/** Unordered of default language dictionary */
std::unordered_map<std::string, std::string> default_dict;
/** Unordered map of current language dict, application currently doesn't have function to use this 
 * (can't set current language during runtime) */
std::unordered_map<std::string, std::string> current_dict;
const char* err_string = "ERR_TXT";

const char* get_localized_text(const char *key) {
    app_workspace *app_wrk = app_workspace::get_instance().get();

    try {
        if (app_wrk->main_config->lang_current.compare(app_wrk->main_config->lang_default))
            // originally wanted this branch to fall back on default_dict in case a key isn't found in
            // current_dict but for simplicity this isn't implemented
            return current_dict.at(key).c_str();
        else
            return default_dict.at(key).c_str();
    } catch (std::out_of_range &ex) {
        spdlog::info("localisation.cpp - Failed to find localisation {0} key for currently selected language."
            "Using key instead.", key);
    }

    //return err_string;
    return key; //return string key instead of general error, to know what string is missing from lang file
}

int load_default_dict() {
    app_workspace *app_wrk = app_workspace::get_instance().get();
    std::string file = app_wrk->main_config->lang_path + "dictionary_" + app_wrk->main_config->lang_default;
    std::string line;
    int sep_index;
    std::ifstream dict(file, std::ios::in);

    if (!dict.is_open()) {
        spdlog::critical("localisation.cpp - Failed to load default language dictionary");
        return 1;
    }

    while (std::getline(dict, line)) {
        sep_index = line.find_first_of('=');
        
        std::string key = line.substr(0, sep_index);
        trim(key);
        std::string val = line.substr(sep_index + 1, line.length());
        trim(val);

        default_dict.emplace(key, val);
    }

    dict.close();
    app_wrk->init_labels();
    return 0;
}

int load_current_dict() {
    app_workspace *app_wrk = app_workspace::get_instance().get();
    std::string file = app_wrk->main_config->lang_path + "dictionary_" + app_wrk->main_config->lang_current;
    std::string line;
    int sep_index;
    std::ifstream dict(file, std::ios::in);

    if (!dict.is_open()) {
        spdlog::critical("localisation.cpp - Failed to load current language dictionary");
        return 1;
    }

    while (std::getline(dict, line)) {
        sep_index = line.find_first_of('=');

        std::string key = line.substr(0, sep_index);
        trim(key);
        std::string val = line.substr(sep_index + 1, line.length());
        trim(val);

        current_dict.emplace(key, val);
    }

    dict.close();
    app_wrk->init_labels();
    return 0;
}